	name    args
	page    55,80

	title   "ARGS.ASM -- routine to decode command line arguments
;
; Copyright 1998 (C), by Michael Radwick, N8XKZ
;

_TEXT   segment para public 'CODE'
	assume cs:_TEXT, ds:_TEXT, es:_TEXT, ss:_TEXT

	PUBLIC getargs

cr      equ     0dh
lf      equ     0ah
colon   equ     ':'
blank   equ     ' '

badopt          db      'Invalid Option: ', '$'
crlf            db      cr,lf,'$'

;
; intrinsic - Decode a single command option and invoke handler
;
; At Entry
;       SI: Point to start of option table
;       DI: Point to command line text
;
input_ptr       dw      0

intrinsic proc near
	mov     input_ptr,di            ;save pointer to text
intr1: 
	cmp     byte ptr [si],0         ;Is command table exhausted?
	je      intr7                   ;jump, end of table.
	mov     di,input_ptr
intr2:
	cmp     byte ptr [di],blank     ;scan off any leading blanks
	jne     intr3
	inc     di
	jmp     intr2

intr3:
	mov     al,[si]                 ;get next command char.
	or      al,al                   ;check if end-of-string
	jz      intr4                   ;jump, entire string matched.
	cmp     al,[di]                 ;compare to next input char.
	jnz     intr6                   ;jump, found mismatch
	inc     si
	inc     di
	jmp     intr3

intr4:
	cmp     byte ptr [di],colon     ;Make sure user's command is
	jne     intr4a                  ;same length as command.
	inc     di                      ;let DI point to option value.
	jmp     intr5
intr4a:
	cmp     byte ptr [di],0
	jne     intr6

intr5:
	call    word ptr [si+1]         ;run the command routine
	ret

intr6:
	lodsb                           ;look for end of this command string.
	or      al,al
	jnz     intr6
	add     si,2                    ;skip over command routine pointer.
	jmp     intr1                   ;try to match next command

intr7:
	mov     ah,9                    ;Output error message
	mov     dx,offset badopt
	int     21h
	mov     si,input_ptr
intr7a:
	lodsb                           ;Add text of bad option
	or      al,al
	jnz     intr7a
	mov     byte ptr [si],'$'
	mov     ah,9
	mov     dx,input_ptr
	int     21h
	mov     ah,9                    ;print cr/lf
	mov     dx,offset crlf
	int     21h
	ret

intrinsic endp

;
; getargs - parse command line arguments
;
; At Entry:
;       SI: pointer to argument table
;       DI: pointer to command line
;
; At Exit:
;       Nothing
;
; The argument table contains a zero terminated string, followed
; by the word address of the function handling the argument.
; The string contains the name of a command-line option.
;
; The function is called with DI pointing to a zero-terminated
; string containing the value of the option.
;
cmdLine         dw      0
currToken       dw      0
cmdTable        dw      0

getargs proc near
	mov     cmdLine,di
	mov     cmdTable,si
getarg1:
	mov     si,cmdLine
getarg2:
	mov     al,[si]         ;skip leading blanks
	cmp     al,blank
	jnz     getarg2a
	inc     si
	jmp     getarg2
;
getarg2a:
	mov     cmdLine,si      ;save si as start of token
	or      al,al           ;see if we are done
	jz      getarg4
getarg3:
	mov     al,[si]
	or      al,al           ;see if we are done
	jz      getarg4
	cmp     al,blank        ;see if we have the whole token
	jz      getarg4
	cmp     al,'a'          ;fold lowercase to uppercase
	jb      getarg3a
	cmp     al,'z'
	ja      getarg3a
	sub     byte ptr [si],'a'-'A'
getarg3a:
	inc     si
	jmp     getarg3
;
getarg4:
	cmp     si,cmdLine
	je      getarg_done
	or      al,al
	jz      getarg4a
	mov     byte ptr [si],0         ;terminate token with end-of-string
	inc     si                      ;point to next token
getarg4a:
	mov     di,cmdLine              ;get start of token
	mov     cmdLine,si              ;save start of next token
	mov     si,cmdTable             ;recover pointer to option table.
	call    intrinsic               ;handle the option.
	jmp     getarg1                 ;try the next option
getarg_done:
	ret

getargs endp

_TEXT   ends
	end

